/*
 * Quasseldroid - Quassel client for Android
 *
 * Copyright (c) 2021 Janne Mareike Koschinski
 * Copyright (c) 2021 The Quassel Project
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package de.justjanne.libquassel.protocol.serializers.handshake

import de.justjanne.libquassel.protocol.features.FeatureSet
import de.justjanne.libquassel.protocol.models.HandshakeMessage
import de.justjanne.libquassel.protocol.testutil.byteBufferOf
import de.justjanne.libquassel.protocol.testutil.handshakeSerializerTest
import org.junit.jupiter.api.Tag
import org.junit.jupiter.api.Test

@Tag("HandshakeSerializerTest")
class ClientInitSerializerTest {
  @Test
  fun testEmptyMap() = handshakeSerializerTest(
    HandshakeMessage.ClientInit(
      clientVersion = null,
      buildDate = null,
      featureSet = FeatureSet.none()
    ),
    byteBufferOf(
      // 4 elements
      0x00u, 0x00u, 0x00u, 0x02u,
      // ByteBuffer
      0x00u, 0x00u, 0x00u, 0x0Cu,
      0x00u,
      // 7 bytes
      0x00u, 0x00u, 0x00u, 0x07u,
      // MsgType
      0x4Du, 0x73u, 0x67u, 0x54u, 0x79u, 0x70u, 0x65u,
      // String
      0x00u, 0x00u, 0x00u, 0x0Au,
      0x00u,
      // 10 2-byte chars
      0x00u, 0x00u, 0x00u, 0x14u,
      0x00u, 0x43u, 0x00u, 0x6Cu, 0x00u, 0x69u, 0x00u, 0x65u, 0x00u, 0x6Eu, 0x00u, 0x74u, 0x00u, 0x49u, 0x00u, 0x6Eu,
      0x00u, 0x69u, 0x00u, 0x74u,
    ),
    featureSets = emptyList(),
    serializeFeatureSet = null
  )

  @Test
  fun testSimple() = handshakeSerializerTest(
    HandshakeMessage.ClientInit(
      clientVersion = "Quasseldroid test",
      buildDate = "Never",
      featureSet = FeatureSet.none()
    ),
    byteBufferOf(
      0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x0Cu,
      0x00u, 0x00u, 0x00u, 0x00u, 0x07u, 0x4Du, 0x73u, 0x67u,
      0x54u, 0x79u, 0x70u, 0x65u, 0x00u, 0x00u, 0x00u, 0x0Au,
      0x00u, 0x00u, 0x00u, 0x00u, 0x14u, 0x00u, 0x43u, 0x00u,
      0x6Cu, 0x00u, 0x69u, 0x00u, 0x65u, 0x00u, 0x6Eu, 0x00u,
      0x74u, 0x00u, 0x49u, 0x00u, 0x6Eu, 0x00u, 0x69u, 0x00u,
      0x74u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u,
      0x00u, 0x0Du, 0x43u, 0x6Cu, 0x69u, 0x65u, 0x6Eu, 0x74u,
      0x56u, 0x65u, 0x72u, 0x73u, 0x69u, 0x6Fu, 0x6Eu, 0x00u,
      0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x22u,
      0x00u, 0x51u, 0x00u, 0x75u, 0x00u, 0x61u, 0x00u, 0x73u,
      0x00u, 0x73u, 0x00u, 0x65u, 0x00u, 0x6Cu, 0x00u, 0x64u,
      0x00u, 0x72u, 0x00u, 0x6Fu, 0x00u, 0x69u, 0x00u, 0x64u,
      0x00u, 0x20u, 0x00u, 0x74u, 0x00u, 0x65u, 0x00u, 0x73u,
      0x00u, 0x74u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u,
      0x00u, 0x00u, 0x0Au, 0x43u, 0x6Cu, 0x69u, 0x65u, 0x6Eu,
      0x74u, 0x44u, 0x61u, 0x74u, 0x65u, 0x00u, 0x00u, 0x00u,
      0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x4Eu,
      0x00u, 0x65u, 0x00u, 0x76u, 0x00u, 0x65u, 0x00u, 0x72u,
      0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u,
      0x08u, 0x46u, 0x65u, 0x61u, 0x74u, 0x75u, 0x72u, 0x65u,
      0x73u, 0x00u, 0x00u, 0x00u, 0x03u, 0x00u, 0x00u, 0x00u,
      0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u,
      0x00u, 0x00u, 0x0Bu, 0x46u, 0x65u, 0x61u, 0x74u, 0x75u,
      0x72u, 0x65u, 0x4Cu, 0x69u, 0x73u, 0x74u, 0x00u, 0x00u,
      0x00u, 0x0Bu, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u
    )
  )

  @Test
  fun testRealistic() = handshakeSerializerTest(
    HandshakeMessage.ClientInit(
      clientVersion = "Quasseldroid <a href=\"https://git.kuschku.de/justJanne/QuasselDroid-ng/commit/" +
        "b622ad63056b6054b06e09f8e1f1ef2b0c3aaf9a\">v1.3.3</a>",
      buildDate = "2020-04-27T22:21:17Z",
      featureSet = FeatureSet.all()
    ),
    byteBufferOf(
      0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x07u, 0x4Du, 0x73u, 0x67u,
      0x54u, 0x79u, 0x70u, 0x65u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x14u, 0x00u, 0x43u, 0x00u,
      0x6Cu, 0x00u, 0x69u, 0x00u, 0x65u, 0x00u, 0x6Eu, 0x00u, 0x74u, 0x00u, 0x49u, 0x00u, 0x6Eu, 0x00u, 0x69u, 0x00u,
      0x74u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Du, 0x43u, 0x6Cu, 0x69u, 0x65u, 0x6Eu, 0x74u,
      0x56u, 0x65u, 0x72u, 0x73u, 0x69u, 0x6Fu, 0x6Eu, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x01u, 0x04u,
      0x00u, 0x51u, 0x00u, 0x75u, 0x00u, 0x61u, 0x00u, 0x73u, 0x00u, 0x73u, 0x00u, 0x65u, 0x00u, 0x6Cu, 0x00u, 0x64u,
      0x00u, 0x72u, 0x00u, 0x6Fu, 0x00u, 0x69u, 0x00u, 0x64u, 0x00u, 0x20u, 0x00u, 0x3Cu, 0x00u, 0x61u, 0x00u, 0x20u,
      0x00u, 0x68u, 0x00u, 0x72u, 0x00u, 0x65u, 0x00u, 0x66u, 0x00u, 0x3Du, 0x00u, 0x22u, 0x00u, 0x68u, 0x00u, 0x74u,
      0x00u, 0x74u, 0x00u, 0x70u, 0x00u, 0x73u, 0x00u, 0x3Au, 0x00u, 0x2Fu, 0x00u, 0x2Fu, 0x00u, 0x67u, 0x00u, 0x69u,
      0x00u, 0x74u, 0x00u, 0x2Eu, 0x00u, 0x6Bu, 0x00u, 0x75u, 0x00u, 0x73u, 0x00u, 0x63u, 0x00u, 0x68u, 0x00u, 0x6Bu,
      0x00u, 0x75u, 0x00u, 0x2Eu, 0x00u, 0x64u, 0x00u, 0x65u, 0x00u, 0x2Fu, 0x00u, 0x6Au, 0x00u, 0x75u, 0x00u, 0x73u,
      0x00u, 0x74u, 0x00u, 0x4Au, 0x00u, 0x61u, 0x00u, 0x6Eu, 0x00u, 0x6Eu, 0x00u, 0x65u, 0x00u, 0x2Fu, 0x00u, 0x51u,
      0x00u, 0x75u, 0x00u, 0x61u, 0x00u, 0x73u, 0x00u, 0x73u, 0x00u, 0x65u, 0x00u, 0x6Cu, 0x00u, 0x44u, 0x00u, 0x72u,
      0x00u, 0x6Fu, 0x00u, 0x69u, 0x00u, 0x64u, 0x00u, 0x2Du, 0x00u, 0x6Eu, 0x00u, 0x67u, 0x00u, 0x2Fu, 0x00u, 0x63u,
      0x00u, 0x6Fu, 0x00u, 0x6Du, 0x00u, 0x6Du, 0x00u, 0x69u, 0x00u, 0x74u, 0x00u, 0x2Fu, 0x00u, 0x62u, 0x00u, 0x36u,
      0x00u, 0x32u, 0x00u, 0x32u, 0x00u, 0x61u, 0x00u, 0x64u, 0x00u, 0x36u, 0x00u, 0x33u, 0x00u, 0x30u, 0x00u, 0x35u,
      0x00u, 0x36u, 0x00u, 0x62u, 0x00u, 0x36u, 0x00u, 0x30u, 0x00u, 0x35u, 0x00u, 0x34u, 0x00u, 0x62u, 0x00u, 0x30u,
      0x00u, 0x36u, 0x00u, 0x65u, 0x00u, 0x30u, 0x00u, 0x39u, 0x00u, 0x66u, 0x00u, 0x38u, 0x00u, 0x65u, 0x00u, 0x31u,
      0x00u, 0x66u, 0x00u, 0x31u, 0x00u, 0x65u, 0x00u, 0x66u, 0x00u, 0x32u, 0x00u, 0x62u, 0x00u, 0x30u, 0x00u, 0x63u,
      0x00u, 0x33u, 0x00u, 0x61u, 0x00u, 0x61u, 0x00u, 0x66u, 0x00u, 0x39u, 0x00u, 0x61u, 0x00u, 0x22u, 0x00u, 0x3Eu,
      0x00u, 0x76u, 0x00u, 0x31u, 0x00u, 0x2Eu, 0x00u, 0x33u, 0x00u, 0x2Eu, 0x00u, 0x33u, 0x00u, 0x3Cu, 0x00u, 0x2Fu,
      0x00u, 0x61u, 0x00u, 0x3Eu, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x43u, 0x6Cu, 0x69u,
      0x65u, 0x6Eu, 0x74u, 0x44u, 0x61u, 0x74u, 0x65u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x28u,
      0x00u, 0x32u, 0x00u, 0x30u, 0x00u, 0x32u, 0x00u, 0x30u, 0x00u, 0x2Du, 0x00u, 0x30u, 0x00u, 0x34u, 0x00u, 0x2Du,
      0x00u, 0x32u, 0x00u, 0x37u, 0x00u, 0x54u, 0x00u, 0x32u, 0x00u, 0x32u, 0x00u, 0x3Au, 0x00u, 0x32u, 0x00u, 0x31u,
      0x00u, 0x3Au, 0x00u, 0x31u, 0x00u, 0x37u, 0x00u, 0x5Au, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u,
      0x08u, 0x46u, 0x65u, 0x61u, 0x74u, 0x75u, 0x72u, 0x65u, 0x73u, 0x00u, 0x00u, 0x00u, 0x03u, 0x00u, 0x00u, 0x00u,
      0xFFu, 0xFFu, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Bu, 0x46u, 0x65u, 0x61u, 0x74u, 0x75u,
      0x72u, 0x65u, 0x4Cu, 0x69u, 0x73u, 0x74u, 0x00u, 0x00u, 0x00u, 0x0Bu, 0x00u, 0x00u, 0x00u, 0x00u, 0x16u, 0x00u,
      0x00u, 0x00u, 0x2Cu, 0x00u, 0x53u, 0x00u, 0x79u, 0x00u, 0x6Eu, 0x00u, 0x63u, 0x00u, 0x68u, 0x00u, 0x72u, 0x00u,
      0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x69u, 0x00u, 0x7Au, 0x00u, 0x65u, 0x00u, 0x64u, 0x00u, 0x4Du, 0x00u, 0x61u, 0x00u,
      0x72u, 0x00u, 0x6Bu, 0x00u, 0x65u, 0x00u, 0x72u, 0x00u, 0x4Cu, 0x00u, 0x69u, 0x00u, 0x6Eu, 0x00u, 0x65u, 0x00u,
      0x00u, 0x00u, 0x24u, 0x00u, 0x53u, 0x00u, 0x61u, 0x00u, 0x73u, 0x00u, 0x6Cu, 0x00u, 0x41u, 0x00u, 0x75u, 0x00u,
      0x74u, 0x00u, 0x68u, 0x00u, 0x65u, 0x00u, 0x6Eu, 0x00u, 0x74u, 0x00u, 0x69u, 0x00u, 0x63u, 0x00u, 0x61u, 0x00u,
      0x74u, 0x00u, 0x69u, 0x00u, 0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x00u, 0x00u, 0x18u, 0x00u, 0x53u, 0x00u, 0x61u, 0x00u,
      0x73u, 0x00u, 0x6Cu, 0x00u, 0x45u, 0x00u, 0x78u, 0x00u, 0x74u, 0x00u, 0x65u, 0x00u, 0x72u, 0x00u, 0x6Eu, 0x00u,
      0x61u, 0x00u, 0x6Cu, 0x00u, 0x00u, 0x00u, 0x28u, 0x00u, 0x48u, 0x00u, 0x69u, 0x00u, 0x64u, 0x00u, 0x65u, 0x00u,
      0x49u, 0x00u, 0x6Eu, 0x00u, 0x61u, 0x00u, 0x63u, 0x00u, 0x74u, 0x00u, 0x69u, 0x00u, 0x76u, 0x00u, 0x65u, 0x00u,
      0x4Eu, 0x00u, 0x65u, 0x00u, 0x74u, 0x00u, 0x77u, 0x00u, 0x6Fu, 0x00u, 0x72u, 0x00u, 0x6Bu, 0x00u, 0x73u, 0x00u,
      0x00u, 0x00u, 0x1Cu, 0x00u, 0x50u, 0x00u, 0x61u, 0x00u, 0x73u, 0x00u, 0x73u, 0x00u, 0x77u, 0x00u, 0x6Fu, 0x00u,
      0x72u, 0x00u, 0x64u, 0x00u, 0x43u, 0x00u, 0x68u, 0x00u, 0x61u, 0x00u, 0x6Eu, 0x00u, 0x67u, 0x00u, 0x65u, 0x00u,
      0x00u, 0x00u, 0x1Cu, 0x00u, 0x43u, 0x00u, 0x61u, 0x00u, 0x70u, 0x00u, 0x4Eu, 0x00u, 0x65u, 0x00u, 0x67u, 0x00u,
      0x6Fu, 0x00u, 0x74u, 0x00u, 0x69u, 0x00u, 0x61u, 0x00u, 0x74u, 0x00u, 0x69u, 0x00u, 0x6Fu, 0x00u, 0x6Eu, 0x00u,
      0x00u, 0x00u, 0x1Eu, 0x00u, 0x56u, 0x00u, 0x65u, 0x00u, 0x72u, 0x00u, 0x69u, 0x00u, 0x66u, 0x00u, 0x79u, 0x00u,
      0x53u, 0x00u, 0x65u, 0x00u, 0x72u, 0x00u, 0x76u, 0x00u, 0x65u, 0x00u, 0x72u, 0x00u, 0x53u, 0x00u, 0x53u, 0x00u,
      0x4Cu, 0x00u, 0x00u, 0x00u, 0x20u, 0x00u, 0x43u, 0x00u, 0x75u, 0x00u, 0x73u, 0x00u, 0x74u, 0x00u, 0x6Fu, 0x00u,
      0x6Du, 0x00u, 0x52u, 0x00u, 0x61u, 0x00u, 0x74u, 0x00u, 0x65u, 0x00u, 0x4Cu, 0x00u, 0x69u, 0x00u, 0x6Du, 0x00u,
      0x69u, 0x00u, 0x74u, 0x00u, 0x73u, 0x00u, 0x00u, 0x00u, 0x1Eu, 0x00u, 0x44u, 0x00u, 0x63u, 0x00u, 0x63u, 0x00u,
      0x46u, 0x00u, 0x69u, 0x00u, 0x6Cu, 0x00u, 0x65u, 0x00u, 0x54u, 0x00u, 0x72u, 0x00u, 0x61u, 0x00u, 0x6Eu, 0x00u,
      0x73u, 0x00u, 0x66u, 0x00u, 0x65u, 0x00u, 0x72u, 0x00u, 0x00u, 0x00u, 0x26u, 0x00u, 0x41u, 0x00u, 0x77u, 0x00u,
      0x61u, 0x00u, 0x79u, 0x00u, 0x46u, 0x00u, 0x6Fu, 0x00u, 0x72u, 0x00u, 0x6Du, 0x00u, 0x61u, 0x00u, 0x74u, 0x00u,
      0x54u, 0x00u, 0x69u, 0x00u, 0x6Du, 0x00u, 0x65u, 0x00u, 0x73u, 0x00u, 0x74u, 0x00u, 0x61u, 0x00u, 0x6Du, 0x00u,
      0x70u, 0x00u, 0x00u, 0x00u, 0x1Cu, 0x00u, 0x41u, 0x00u, 0x75u, 0x00u, 0x74u, 0x00u, 0x68u, 0x00u, 0x65u, 0x00u,
      0x6Eu, 0x00u, 0x74u, 0x00u, 0x69u, 0x00u, 0x63u, 0x00u, 0x61u, 0x00u, 0x74u, 0x00u, 0x6Fu, 0x00u, 0x72u, 0x00u,
      0x73u, 0x00u, 0x00u, 0x00u, 0x24u, 0x00u, 0x42u, 0x00u, 0x75u, 0x00u, 0x66u, 0x00u, 0x66u, 0x00u, 0x65u, 0x00u,
      0x72u, 0x00u, 0x41u, 0x00u, 0x63u, 0x00u, 0x74u, 0x00u, 0x69u, 0x00u, 0x76u, 0x00u, 0x69u, 0x00u, 0x74u, 0x00u,
      0x79u, 0x00u, 0x53u, 0x00u, 0x79u, 0x00u, 0x6Eu, 0x00u, 0x63u, 0x00u, 0x00u, 0x00u, 0x24u, 0x00u, 0x43u, 0x00u,
      0x6Fu, 0x00u, 0x72u, 0x00u, 0x65u, 0x00u, 0x53u, 0x00u, 0x69u, 0x00u, 0x64u, 0x00u, 0x65u, 0x00u, 0x48u, 0x00u,
      0x69u, 0x00u, 0x67u, 0x00u, 0x68u, 0x00u, 0x6Cu, 0x00u, 0x69u, 0x00u, 0x67u, 0x00u, 0x68u, 0x00u, 0x74u, 0x00u,
      0x73u, 0x00u, 0x00u, 0x00u, 0x1Cu, 0x00u, 0x53u, 0x00u, 0x65u, 0x00u, 0x6Eu, 0x00u, 0x64u, 0x00u, 0x65u, 0x00u,
      0x72u, 0x00u, 0x50u, 0x00u, 0x72u, 0x00u, 0x65u, 0x00u, 0x66u, 0x00u, 0x69u, 0x00u, 0x78u, 0x00u, 0x65u, 0x00u,
      0x73u, 0x00u, 0x00u, 0x00u, 0x20u, 0x00u, 0x52u, 0x00u, 0x65u, 0x00u, 0x6Du, 0x00u, 0x6Fu, 0x00u, 0x74u, 0x00u,
      0x65u, 0x00u, 0x44u, 0x00u, 0x69u, 0x00u, 0x73u, 0x00u, 0x63u, 0x00u, 0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x6Eu, 0x00u,
      0x65u, 0x00u, 0x63u, 0x00u, 0x74u, 0x00u, 0x00u, 0x00u, 0x20u, 0x00u, 0x45u, 0x00u, 0x78u, 0x00u, 0x74u, 0x00u,
      0x65u, 0x00u, 0x6Eu, 0x00u, 0x64u, 0x00u, 0x65u, 0x00u, 0x64u, 0x00u, 0x46u, 0x00u, 0x65u, 0x00u, 0x61u, 0x00u,
      0x74u, 0x00u, 0x75u, 0x00u, 0x72u, 0x00u, 0x65u, 0x00u, 0x73u, 0x00u, 0x00u, 0x00u, 0x10u, 0x00u, 0x4Cu, 0x00u,
      0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x67u, 0x00u, 0x54u, 0x00u, 0x69u, 0x00u, 0x6Du, 0x00u, 0x65u, 0x00u, 0x00u, 0x00u,
      0x18u, 0x00u, 0x52u, 0x00u, 0x69u, 0x00u, 0x63u, 0x00u, 0x68u, 0x00u, 0x4Du, 0x00u, 0x65u, 0x00u, 0x73u, 0x00u,
      0x73u, 0x00u, 0x61u, 0x00u, 0x67u, 0x00u, 0x65u, 0x00u, 0x73u, 0x00u, 0x00u, 0x00u, 0x22u, 0x00u, 0x42u, 0x00u,
      0x61u, 0x00u, 0x63u, 0x00u, 0x6Bu, 0x00u, 0x6Cu, 0x00u, 0x6Fu, 0x00u, 0x67u, 0x00u, 0x46u, 0x00u, 0x69u, 0x00u,
      0x6Cu, 0x00u, 0x74u, 0x00u, 0x65u, 0x00u, 0x72u, 0x00u, 0x54u, 0x00u, 0x79u, 0x00u, 0x70u, 0x00u, 0x65u, 0x00u,
      0x00u, 0x00u, 0x1Eu, 0x00u, 0x45u, 0x00u, 0x63u, 0x00u, 0x64u, 0x00u, 0x73u, 0x00u, 0x61u, 0x00u, 0x43u, 0x00u,
      0x65u, 0x00u, 0x72u, 0x00u, 0x74u, 0x00u, 0x66u, 0x00u, 0x70u, 0x00u, 0x4Bu, 0x00u, 0x65u, 0x00u, 0x79u, 0x00u,
      0x73u, 0x00u, 0x00u, 0x00u, 0x1Au, 0x00u, 0x4Cu, 0x00u, 0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x67u, 0x00u, 0x4Du, 0x00u,
      0x65u, 0x00u, 0x73u, 0x00u, 0x73u, 0x00u, 0x61u, 0x00u, 0x67u, 0x00u, 0x65u, 0x00u, 0x49u, 0x00u, 0x64u, 0x00u,
      0x00u, 0x00u, 0x1Cu, 0x00u, 0x53u, 0x00u, 0x79u, 0x00u, 0x6Eu, 0x00u, 0x63u, 0x00u, 0x65u, 0x00u, 0x64u, 0x00u,
      0x43u, 0x00u, 0x6Fu, 0x00u, 0x72u, 0x00u, 0x65u, 0x00u, 0x49u, 0x00u, 0x6Eu, 0x00u, 0x66u, 0x00u, 0x6Fu
    )
  )
}
